
/* Written by Pavel Kos */

/* GNOME */
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define MAX_ROWS 150
#define MAX_COLS 300

char board[MAX_ROWS][MAX_COLS + 1];
int remains[MAX_COLS];
int moves[MAX_COLS];

int rows, cols;

#define NEIGHBOUR(R,C) \
  (((R >= 0) && (R < rows) && (C >= 0) && (C < cols))?board[R][C]:' ')

int Erase(int r, int c, char col)
{
  int ret = 1;

  board[r][c] = ' ';
  --remains[c];
  moves[c] = 1;
  if (NEIGHBOUR(r - 1, c) == col)
    ret += Erase(r - 1, c, col);
  if (NEIGHBOUR(r + 1, c) == col)
    ret += Erase(r + 1, c, col);
  if (NEIGHBOUR(r, c - 1) == col)
    ret += Erase(r, c - 1, col);
  if (NEIGHBOUR(r, c + 1) == col)
    ret += Erase(r, c + 1, col);
  return ret;
}

void Fall(void)
{ /* Provede popadnuti kamenu */
  int i, j, skip;

  for (i = 0; i < cols; ++i)
  {
    if (moves[i])
    {
      j = 0;
      while ((j < remains[i]) && (!isspace(board[j][i])))
        ++j;
      skip = 0;
      while (j < remains[i] + skip)
      {
        if (isspace(board[j][i]))
          ++skip;
        else
        {
          board[j - skip][i] = board[j][i];
          board[j][i] = ' ';
        }
        ++j;
      }
    }
  }
}

void Squeeze(void)
{
  int i, j, skip;

  i = 0;
  while ((i < cols) && (remains[i]))
    ++i;
  skip = 0;
  while (i < cols + skip)
  {
    if (remains[i] == 0)
    {
      ++skip;
      --cols;
    }
    else
    {
      for (j = 0; j < remains[i]; ++j)
      {
        board[j][i - skip] = board[j][i];
        board[j][i] = ' ';
      }
      remains[i - skip] = remains[i];
      remains[i] = 0;
    }
    ++i;
  }
}

int Play(int r, int c)
{
  int ct;
  char col = board[r][c];

  if (isspace(col))
    return 0;
  if ((NEIGHBOUR(r - 1, c) == col) || (NEIGHBOUR(r + 1, c) == col) ||
      (NEIGHBOUR(r, c - 1) == col) || (NEIGHBOUR(r, c + 1) == col))
  {
    memset(moves, 0, sizeof(moves));
    ct = Erase(r, c, col);
    Fall();
    Squeeze();
    return (ct - 2)*(ct - 2);
  }
  return 0;
}

int main(void)
{
  int n, i, r, c, score;

  scanf("%d", &n);
  while (n--)
  {
    scanf("%d%d", &cols, &rows);
    for (i = 0; i < rows; ++i)
      scanf("%s", board[rows - i - 1]);
    for (i = 0; i < cols; ++i)
      remains[i] = rows;
    scanf("%d", &i);
    score = 0;
    while (i--)
    {
      scanf("%d%d", &c, &r);
      score += Play(r - 1, c - 1);
    }
    if (cols == 0)
      score += 1000;
    printf("Game over!\n"
           "Score dosazene v teto hre je %d bodu.\n"
           "Byli bychom radi, kdybyste si zahrali jeste jednou.\n"
           "Prejete si hrat znovu?\n"
           "Prijemnou zabavu Vam preje firma ACMTENDO.\n", score);
  }
  return 0;
}

